@@ -24,13 +24,20 @@
 #define _ARCH_TYPES_H__
 
 /** cpu specific types */
-typedef unsigned int size_t;
 typedef unsigned int irq_flags_t;
 typedef unsigned int virtual_addr_t;
 typedef unsigned int virtual_size_t;
 typedef unsigned int physical_addr_t;
 typedef unsigned int physical_size_t;
 
+#define __ARCH_PRIADDR_PREFIX	""
+#define __ARCH_PRIADDR_DIGITS	"8"
+#define __ARCH_PRISIZE_PREFIX	""
+#define __ARCH_PRIPADDR_PREFIX	""
+#define __ARCH_PRIPADDR_DIGITS	"8"
+#define __ARCH_PRIPSIZE_PREFIX	""
+#define __ARCH_PRI64_PREFIX	"ll"
+
 typedef struct {
 	volatile long counter;
 } atomic_t;
@@ -43,6 +50,18 @@ typedef struct {
 	volatile long lock;
 } arch_spinlock_t;
 
+#define ARCH_ATOMIC_INIT(_lptr, val)		\
+	(_lptr)->counter = (val)
+
+#define ARCH_ATOMIC_INITIALIZER(val)		\
+	{ .counter = (val), }
+
+#define ARCH_ATOMIC64_INIT(_lptr, val)		\
+	(_lptr)->counter = (val)
+
+#define ARCH_ATOMIC64_INITIALIZER(val)		\
+	{ .counter = (val), }
+
 #define __ARCH_SPIN_UNLOCKED		0xffffffff
 
 /* FIXME: Need memory barrier for this. */
