@@ -357,7 +357,7 @@ u32 cpu_vcpu_spsr_retrieve(struct vmm_vcpu *vcpu, u32 mode)
 	u32 hwreg;
 	if (vcpu != vmm_scheduler_current_vcpu()) {
 		/* This function should only be called for current VCPU */
-		vmm_panic("%d not called for current vcpu\n", __func__);
+		vmm_panic("%s not called for current vcpu\n", __func__);
 	}
 	/* Find out correct SPSR */
 	switch (mode) {
@@ -402,7 +402,7 @@ int cpu_vcpu_spsr_update(struct vmm_vcpu *vcpu,
 	}
 	if (vcpu != vmm_scheduler_current_vcpu()) {
 		/* This function should only be called for current VCPU */
-		vmm_panic("%d not called for current vcpu\n", __func__);
+		vmm_panic("%s not called for current vcpu\n", __func__);
 	}
 	/* Update appropriate SPSR */
 	switch (mode) {
@@ -501,7 +501,18 @@ int arch_vcpu_init(struct vmm_vcpu *vcpu)
 	/* For both Orphan & Normal VCPUs */
 	memset(arm_regs(vcpu), 0, sizeof(arch_regs_t));
 	arm_regs(vcpu)->pc = vcpu->start_pc;
-	arm_regs(vcpu)->sp = vcpu->stack_va + vcpu->stack_sz - 4;
+	/*
+	 * Stacks must be 64-bits aligned to respect AAPCS:
+	 * Procedure Call Standard for the ARM Architecture.
+	 * To do so, AAPCS advises that all SP must be set to
+	 * a value which is 0 modulo 8.
+	 * The compiler takes care of the frame size.
+	 *
+	 * This is terribly important because it messes runtime
+	 * with values greater than 32 bits (e.g. 64-bits integers).
+	 */
+	arm_regs(vcpu)->sp = vcpu->stack_va + vcpu->stack_sz - 8;
+	arm_regs(vcpu)->sp = arm_regs(vcpu)->sp & ~0x7;
 	if (vcpu->is_normal) {
 		arm_regs(vcpu)->cpsr  = CPSR_ZERO_MASK;
 		arm_regs(vcpu)->cpsr |= CPSR_ASYNC_ABORT_DISABLED;
