@@ -32,6 +32,8 @@
 #include <arch_sections.h>
 #include <cpu_interrupts.h>
 #include <stacktrace.h>
+#include <arch_guest_helper.h>
+#include <cpu_extables.h>
 
 #undef __DEBUG
 //#define __DEBUG
@@ -53,6 +55,16 @@ extern struct tss64_desc __xvisor_tss_64_desc;
 				pa; \
 				})
 
+void reload_host_tss(void)
+{
+	struct tss64_desc *tss_64_desc = &__xvisor_tss_64_desc;
+
+	tss_64_desc->tbt.bits.type = _GATE_TYPE_TSS_AVAILABLE;
+
+	__asm__ volatile("ltr %w0\n\t"
+			 ::"q"(VMM_TSS_SEG_SEL));
+}
+
 static int install_idt(void)
 {
 	memset(&int_desc_table, 0, sizeof(int_desc_table));
@@ -185,7 +197,7 @@ static void setup_gate_handlers(void)
 	set_trap_gate(19, VIRT_TO_PHYS(_exception_simd_err), EXCEPTION_STACK);		/* simd coproc error */
 
 	set_interrupt_gate(2, VIRT_TO_PHYS(_exception_nmi), NMI_STACK);			/* NMI */
-	set_interrupt_gate(14, VIRT_TO_PHYS(_exception_page_fault), REGULAR_INT_STACK);	/* page fault */
+	set_interrupt_gate(14, VIRT_TO_PHYS(_exception_page_fault), DEBUG_STACK);	/* page fault */
 }
 
 int __cpuinit arch_cpu_irq_setup(void)
@@ -213,9 +225,8 @@ extern void print_stacktrace(struct stack_trace *trace);
 		trace.skip = 0;					\
 								\
 		dump_vcpu_regs(regs);					\
+                vmm_printf("\n");                                       \
 		arch_save_stacktrace_regs(regs, &trace);		\
-									\
-		vmm_printf("\n");					\
 		vmm_printf("call trace:\n");				\
 		print_stacktrace(&trace);				\
 	} while(0);
@@ -229,17 +240,21 @@ int do_page_fault(int error, arch_regs_t *regs)
 	__asm__ __volatile__("movq %%cr2, %0\n\t"
 			     :"=r"(bad_vaddr));
 
-	vmm_printf("\n\n");
-	cvcpu = vmm_scheduler_current_vcpu();
+        if (!fixup_exception(regs)) {
+                vmm_printf("\n\n");
+                cvcpu = vmm_scheduler_current_vcpu();
 
-	if (cvcpu) {
-		do_panic_dump(regs, "Unhandled access from VMM vcpu %s @ address %lx\n",
-			cvcpu->name, bad_vaddr);
-	} else {
-		do_panic_dump(regs, "(Page Fault): Unhandled VMM access to address %lx\n", bad_vaddr);
-	}
+                if (cvcpu) {
+                        do_panic_dump(regs, "Unhandled access from VMM vcpu %s @ address %lx\n",
+                                      cvcpu->name, bad_vaddr);
+                } else {
+                        do_panic_dump(regs, "(Page Fault): Unhandled VMM access to address %lx\n", bad_vaddr);
+                }
 
-	while(1); /* Should never reach here. */
+                while(1); /* Should never reach here. */
+        }
+
+        return VMM_OK;
 }
 
 int do_breakpoint(int intno, arch_regs_t *regs)
@@ -248,23 +263,52 @@ int do_breakpoint(int intno, arch_regs_t *regs)
 	while(1);
 }
 
+int do_generic_exception_handler(int intno, arch_regs_t *regs)
+{
+        if (!fixup_exception(regs)) {
+                vmm_printf("Unhandled exception %d\n", intno);
+                do_panic_dump(regs, "Unhandled exception in VMM code.\n");
+                while(1);
+        }
+
+        return VMM_OK;
+}
+
 int do_gpf(int intno, arch_regs_t *regs)
 {
-	do_panic_dump(regs, "(General Proctection Fault)\n");
+        if (!fixup_exception(regs)) {
+                do_panic_dump(regs, "(General Proctection Fault)\n");
 
-	while(1); /* Should never reach here. */
+                while(1); /* Should never reach here. */
+        }
+
+        return VMM_OK;
 }
 
 int do_generic_int_handler(int intno, arch_regs_t *regs)
 {
+	struct vmm_guest *guest;
+	struct vcpu_hw_context *context = NULL;
+
 	if (intno == 0x80) {
-		vmm_scheduler_preempt_orphan(regs);
+		if (regs->rdi == GUEST_HALT_SW_CODE) {
+			guest = (struct vmm_guest *)regs->rsi;
+			vmm_manager_guest_halt(guest);
+		} else if (regs->rdi == GUEST_VM_EXIT_SW_CODE) {
+			context = (struct vcpu_hw_context *)regs->rsi;
+
+			vmm_scheduler_irq_enter(regs, TRUE);
+			context->vcpu_exit(context);
+			vmm_scheduler_irq_exit(regs);
+		} else {
+			vmm_scheduler_preempt_orphan(regs);
+		}
 	} else {
 		/* Get interrupt number from Vector number */
 		intno -= USER_DEFINED_IRQ_BASE;
 
 		vmm_scheduler_irq_enter(regs, FALSE);
-		vmm_host_irq_exec(intno);
+		vmm_host_generic_irq_exec(intno);
 		vmm_scheduler_irq_exit(regs);
 	}
 
