@@ -303,7 +303,7 @@ static int cmd_vfs_ls(struct vmm_chardev *cdev, const char *path)
 			type[9] = 'x';
 		}
 		vmm_cprintf(cdev, "%10s ", type);
-		vmm_cprintf(cdev, "%10ll ", st.st_size);
+		vmm_cprintf(cdev, "%10lld ", st.st_size);
 		vmm_wallclock_mkinfo(st.st_mtime, 0, &ti);
 		switch (ti.tm_mon) {
 		case 0:
@@ -345,7 +345,7 @@ static int cmd_vfs_ls(struct vmm_chardev *cdev, const char *path)
 		default:
 			break;
 		};
-		vmm_cprintf(cdev, "%02d %d %02d:%02d:%02d ",
+		vmm_cprintf(cdev, "%02d %ld %02d:%02d:%02d ",
 				  ti.tm_mday, ti.tm_year + 1900,
 				  ti.tm_hour, ti.tm_min, ti.tm_sec);
 		if (type[0] == 'd')
@@ -708,14 +708,14 @@ static int cmd_vfs_module_load(struct vmm_chardev *cdev, const char *path)
 	}
 
 	if (!len) {
-		vmm_cprintf(cdev, "File %s has zero %d bytes.\n", path);
+		vmm_cprintf(cdev, "File %s has zero bytes.\n", path);
 		rc = VMM_EINVALID;
 		goto fail_closefd;
 	}
 
 	if (len > VFS_MAX_MODULE_SZ) {
-		vmm_cprintf(cdev, "File %s has size %d bytes (> %d bytes).\n",
-			    path, (long)len, VFS_MAX_MODULE_SZ);
+		vmm_cprintf(cdev, "File %s has size %"PRId32" bytes (> %d bytes).\n",
+			    path, len, VFS_MAX_MODULE_SZ);
 		rc = VMM_EINVALID;
 		goto fail_closefd;
 	}
@@ -786,14 +786,14 @@ static int cmd_vfs_fdt_load(struct vmm_chardev *cdev,
 	}
 
 	if (!len) {
-		vmm_cprintf(cdev, "File %s has zero %d bytes.\n", path);
+		vmm_cprintf(cdev, "File %s has zero bytes.\n", path);
 		rc = VMM_EINVALID;
 		goto fail_closefd;
 	}
 
 	if (len > VFS_MAX_FDT_SZ) {
-		vmm_cprintf(cdev, "File %s has size %d bytes (> %d bytes).\n",
-			    path, (long)len, VFS_MAX_FDT_SZ);
+		vmm_cprintf(cdev, "File %s has size %"PRId32" bytes (> %d bytes).\n",
+			    path, len, VFS_MAX_FDT_SZ);
 		rc = VMM_EINVALID;
 		goto fail_closefd;
 	}
@@ -1033,8 +1033,8 @@ static int cmd_vfs_load(struct vmm_chardev *cdev,
 		buf_count = vfs_read(fd, buf, buf_rd);
 		if (buf_count < 1) {
 			vmm_cprintf(cdev, "Failed to read "
-					  "%d bytes @ 0x%llx from %s\n",
-					   buf_rd, (u64)rd_off, path);
+					  "%zu bytes @ 0x%llx from %s\n",
+					   buf_rd, rd_off, path);
 			break;
 		}
 		rd_off += buf_count;
@@ -1047,8 +1047,8 @@ static int cmd_vfs_load(struct vmm_chardev *cdev,
 		}
 		if (buf_wr != buf_count) {
 			vmm_cprintf(cdev, "Failed to write "
-					  "%d bytes @ 0x%llx (%s)\n",
-					  buf_count, (u64)wr_pa,
+					  "%zu bytes @ 0x%"PRIPADDR" (%s)\n",
+					  buf_count, wr_pa,
 					  (guest) ? (guest->name) : "host");
 			break;
 		}
@@ -1057,9 +1057,9 @@ static int cmd_vfs_load(struct vmm_chardev *cdev,
 		wr_pa += buf_wr;
 	}
 
-	vmm_cprintf(cdev, "%s: Loaded 0x%llx with %d bytes\n",
+	vmm_cprintf(cdev, "%s: Loaded 0x%"PRIPADDR" with %zu bytes\n",
 			  (guest) ? (guest->name) : "host",
-			  (u64)pa, wr_count);
+			  pa, wr_count);
 
 	vmm_free(buf);
 	rc = vfs_close(fd);
@@ -1161,9 +1161,9 @@ static int cmd_vfs_load_list(struct vmm_chardev *cdev,
 		pa = (physical_addr_t)strtoull(token, NULL, 0);
 
 		token = cmd_vfs_next_token(&buf, &len);
-		vmm_cprintf(cdev, "%s: Loading 0x%llx with file %s\n",
+		vmm_cprintf(cdev, "%s: Loading 0x%"PRIPADDR" with file %s\n",
 			    (guest) ? (guest->name) : "host",
-			    (u64)pa, token);
+			    pa, token);
 
 		rc = cmd_vfs_load(cdev, guest, pa, token, 0, 0xFFFFFFFF);
 		if (rc) {
