@@ -33,7 +33,7 @@
 #include <libs/bitmap.h>
 
 struct vmm_cpumask { 
-	DEFINE_BITMAP(bits, CONFIG_CPU_COUNT); 
+	DECLARE_BITMAP(bits, CONFIG_CPU_COUNT); 
 };
 
 typedef struct vmm_cpumask vmm_cpumask_t;
@@ -144,6 +144,16 @@ static inline size_t vmm_cpumask_size(void)
 
 #endif
 
+#define VMM_CPU_MASK_NONE						\
+(vmm_cpumask_t) { {							\
+	[0 ... BITS_TO_LONGS(CONFIG_CPU_COUNT)-1] =  0UL		\
+} }
+
+#define VMM_CPU_MASK_CPU0						\
+(vmm_cpumask_t) { {							\
+	[0] =  1UL							\
+} }
+
 /*
  * The following particular system cpumasks and operations manage
  * possible, present, active and online cpus.
@@ -612,7 +622,7 @@ static inline void vmm_cpumask_copy(struct vmm_cpumask *dstp,
 
 /* It's common to want to use cpu_all_mask in struct member initializers,
  * so it has to refer to an address rather than a pointer. */
-extern const DEFINE_BITMAP(cpu_all_bits, CONFIG_CPU_COUNT);
+extern const DECLARE_BITMAP(cpu_all_bits, CONFIG_CPU_COUNT);
 #define cpu_all_mask to_cpumask(cpu_all_bits)
 
 /* First bits of cpu_bit_bitmap are in fact unset. */
