@@ -196,6 +196,7 @@ static struct vmm_vcpu *__vmm_scheduler_next2(struct vmm_scheduler_ctrl *schedp,
 		tcurrent = current;
 	}
 
+dequeue_again:
 	rc = rq_dequeue(schedp, &next, &next_time_slice);
 	if (rc) {
 		/* This should never happen !!! */
@@ -204,6 +205,19 @@ static struct vmm_vcpu *__vmm_scheduler_next2(struct vmm_scheduler_ctrl *schedp,
 
 	if (next != current) {
 		vmm_write_lock_irqsave_lite(&next->sched_lock, nf);
+		/* On SMP host, the next VCPU dequeued from ready queue can
+		 * be RESET, PAUSED, or HALTED from another host CPU after
+		 * it is dequeued and before we acquire next->sched_lock.
+		 *
+		 * To tackle the above situation, we check whether the next
+		 * VCPU is in READY state or not before doing context switch.
+		 * If the next VCPU is not in READY state then we try to
+		 * dequeue another next VCPU.
+		 */
+		if (arch_atomic_read(&next->state) != VMM_VCPU_STATE_READY) {
+			vmm_write_unlock_irqrestore_lite(&next->sched_lock, nf);
+			goto dequeue_again;
+		}
 		arch_vcpu_switch(tcurrent, next, regs);
 	}
 
@@ -222,12 +236,12 @@ static struct vmm_vcpu *__vmm_scheduler_next2(struct vmm_scheduler_ctrl *schedp,
 	return (next != current) ? next : NULL;
 }
 
-/* Must be called with write lock held on current->sched_lock */
-static struct vmm_vcpu *__vmm_scheduler_switch(struct vmm_scheduler_ctrl *schedp,
-						struct vmm_vcpu *current,
-						arch_regs_t *regs)
+static void vmm_scheduler_switch(struct vmm_scheduler_ctrl *schedp,
+				 arch_regs_t *regs)
 {
-	struct vmm_vcpu *next = NULL;
+	u32 preempt_min;
+	struct vmm_vcpu *next;
+	struct vmm_vcpu *current = schedp->current_vcpu;
 
 	if (!regs) {
 		/* This should never happen !!! */
@@ -235,33 +249,32 @@ static struct vmm_vcpu *__vmm_scheduler_switch(struct vmm_scheduler_ctrl *schedp
 	}
 
 	if (current) {
-		if (!current->preempt_count) {
+		preempt_min = (current->wq_lock) ? 1 : 0;
+		if (current->preempt_count == preempt_min) {
+			irq_flags_t cf;
+
+			vmm_write_lock_irqsave_lite(&current->sched_lock, cf);
 			next = __vmm_scheduler_next2(schedp, current, regs);
+			vmm_write_unlock_irqrestore_lite(&current->sched_lock,
+							 cf);
+			if (next != current) {
+				if (current->wq_lock) {
+					vmm_spin_unlock_lite(current->wq_lock);
+					arch_cpu_irq_save(cf);
+					if (current->preempt_count) {
+						current->preempt_count--;
+					}
+					arch_cpu_irq_restore(cf);
+				}
+			}
 		} else {
 			vmm_timer_event_restart(&schedp->ev);
+			next = NULL;
 		}
 	} else {
 		next = __vmm_scheduler_next1(schedp, regs);
 	}
 
-	return next;
-}
-
-static void vmm_scheduler_switch(struct vmm_scheduler_ctrl *schedp,
-				 arch_regs_t *regs)
-{
-	irq_flags_t cf;
-	struct vmm_vcpu *next = NULL;
-	struct vmm_vcpu *current = schedp->current_vcpu;
-
-	if (current) { /* Normal scheduling */
-		vmm_write_lock_irqsave_lite(&current->sched_lock, cf);
-		next = __vmm_scheduler_switch(schedp, current, regs);
-		vmm_write_unlock_irqrestore_lite(&current->sched_lock, cf);
-	} else { /* First time scheduling */
-		next = __vmm_scheduler_switch(schedp, NULL, regs);
-	}
-
 	if (next) {
 		arch_vcpu_post_switch(next, regs);
 	}
@@ -351,13 +364,12 @@ int vmm_scheduler_force_resched(u32 hcpu)
 	return VMM_OK;
 }
 
-int vmm_scheduler_state_change(struct vmm_vcpu *vcpu, u32 new_state,
-			       vmm_spinlock_t *wq_lock)
+int vmm_scheduler_state_change(struct vmm_vcpu *vcpu, u32 new_state)
 {
 	u64 tstamp;
 	int rc = VMM_OK;
 	irq_flags_t flags;
-	bool preempt = FALSE;
+	bool resumed, preempt = FALSE;
 	u32 chcpu = vmm_smp_processor_id(), vhcpu;
 	struct vmm_scheduler_ctrl *schedp;
 	u32 current_state;
@@ -366,7 +378,9 @@ int vmm_scheduler_state_change(struct vmm_vcpu *vcpu, u32 new_state,
 		return VMM_EFAIL;
 	}
 
-	vmm_write_lock_irqsave_lite(&vcpu->sched_lock, flags);
+	arch_cpu_irq_save(flags);
+
+	vmm_write_lock_lite(&vcpu->sched_lock);
 
 	vhcpu = vcpu->hcpu;
 	schedp = &per_cpu(sched, vhcpu);
@@ -422,6 +436,7 @@ int vmm_scheduler_state_change(struct vmm_vcpu *vcpu, u32 new_state,
 			 * resume event while VCPU is RUNNING.
 			 */
 			vcpu->resumed = TRUE;
+			preempt = TRUE;
 			goto skip_state_change;
 		} else if (current_state == VMM_VCPU_STATE_READY) {
 			/* READY->READY is a valid scenario... Do nothing. */
@@ -431,7 +446,7 @@ int vmm_scheduler_state_change(struct vmm_vcpu *vcpu, u32 new_state,
 		}
 		break;
 	case VMM_VCPU_STATE_RUNNING:
-		/* Only scheduler can set RUNNING state.
+		/* Only context-switch can set RUNNING state.
 		 * Any request for setting RUNNING state is invalid.
 		 */
 		rc = VMM_EINVALID;
@@ -443,13 +458,11 @@ int vmm_scheduler_state_change(struct vmm_vcpu *vcpu, u32 new_state,
 	case VMM_VCPU_STATE_HALTED:
 		if ((current_state == VMM_VCPU_STATE_READY) ||
 		    (current_state == VMM_VCPU_STATE_RUNNING)) {
-			if (vcpu->resumed &&
-			    (new_state == VMM_VCPU_STATE_PAUSED)) {
-				vcpu->resumed = FALSE;
-				goto skip_state_change;
-			}
+			resumed = vcpu->resumed;
 			vcpu->resumed = FALSE;
-			if (schedp->current_vcpu == vcpu) {
+			if (resumed && (new_state == VMM_VCPU_STATE_PAUSED)) {
+				goto skip_state_change;
+			} else if (schedp->current_vcpu == vcpu) {
 				/* Preempt current VCPU if paused or halted */
 				preempt = TRUE;
 			} else if (current_state == VMM_VCPU_STATE_READY) {
@@ -496,7 +509,7 @@ int vmm_scheduler_state_change(struct vmm_vcpu *vcpu, u32 new_state,
 	}
 
 skip_state_change:
-	vmm_write_unlock_irqrestore_lite(&vcpu->sched_lock, flags);
+	vmm_write_unlock_lite(&vcpu->sched_lock);
 
 	if (preempt && schedp->current_vcpu) {
 		if (chcpu == vhcpu) {
@@ -505,12 +518,9 @@ skip_state_change:
 			} else if (schedp->irq_context) {
 				vmm_scheduler_switch(schedp, schedp->irq_regs);
 			} else {
-				if (wq_lock) {
-					vmm_spin_unlock(wq_lock);
-				}
 				arch_vcpu_preempt_orphan();
-				if (wq_lock) {
-					vmm_spin_lock(wq_lock);
+				if (vcpu->wq_lock) {
+					vmm_spin_lock(vcpu->wq_lock);
 				}
 			}
 		} else {
@@ -518,6 +528,8 @@ skip_state_change:
 		}
 	}
 
+	arch_cpu_irq_restore(flags);
+
 	if (rc) {
 		vmm_printf("vcpu=%s current_state=0x%x to new_state=0x%x "
 			   "failed (error %d)\n",
@@ -869,6 +881,13 @@ struct vmm_vcpu *vmm_scheduler_current_vcpu(void)
 	return this_cpu(sched).current_vcpu;
 }
 
+u8 vmm_scheduler_current_priority(void)
+{
+	struct vmm_vcpu *cvcpu = vmm_scheduler_current_vcpu();
+
+	return (cvcpu) ? cvcpu->priority : VMM_VCPU_MAX_PRIORITY;
+}
+
 struct vmm_guest *vmm_scheduler_current_guest(void)
 {
 	struct vmm_vcpu *vcpu = this_cpu(sched).current_vcpu;
@@ -878,36 +897,20 @@ struct vmm_guest *vmm_scheduler_current_guest(void)
 
 void vmm_scheduler_yield(void)
 {
-	irq_flags_t flags;
 	struct vmm_scheduler_ctrl *schedp = &this_cpu(sched);
-
-	arch_cpu_irq_save(flags);
+	struct vmm_vcpu *vcpu = this_cpu(sched).current_vcpu;
 
 	if (schedp->irq_context) {
 		vmm_panic("%s: Cannot yield in IRQ context\n", __func__);
 	}
 
-	if (!schedp->current_vcpu) {
+	if (!vcpu) {
 		vmm_panic("%s: NULL VCPU pointer\n", __func__);
 	}
 
-	if (schedp->current_vcpu->is_normal) {
-		/* For Normal VCPU
-		 * Just enable yield on exit and rest will be taken care
-		 * by vmm_scheduler_irq_exit()
-		 */
-		if (vmm_manager_vcpu_get_state(schedp->current_vcpu) ==
-						VMM_VCPU_STATE_RUNNING) {
-			schedp->yield_on_irq_exit = TRUE;
-		}
-	} else {
-		/* For Orphan VCPU
-		 * Forcefully expire yield
-		 */
-		arch_vcpu_preempt_orphan();
+	if (vmm_manager_vcpu_get_state(vcpu) == VMM_VCPU_STATE_RUNNING) {
+		vmm_scheduler_state_change(vcpu, VMM_VCPU_STATE_READY);
 	}
-
-	arch_cpu_irq_restore(flags);
 }
 
 static void idle_orphan(void)
