@@ -62,7 +62,7 @@ int vmm_threads_stop(struct vmm_thread *tinfo)
 		return VMM_EFAIL;
 	}
 
-	if ((rc = vmm_manager_vcpu_halt(tinfo->tvcpu))) {
+	if ((rc = vmm_manager_vcpu_reset(tinfo->tvcpu))) {
 		return rc;
 	}
 
@@ -131,18 +131,20 @@ int vmm_threads_get_name(char *dst, struct vmm_thread *tinfo)
 int vmm_threads_get_state(struct vmm_thread *tinfo)
 {
 	int rc = -1;
+	u32 state;
 
 	if (!tinfo) {
 		rc =  -1;
 	} else {
-		if (tinfo->tvcpu->state & VMM_VCPU_STATE_RESET) { 
+		state = vmm_manager_vcpu_get_state(tinfo->tvcpu);
+		if (state & VMM_VCPU_STATE_RESET) {
 			rc = VMM_THREAD_STATE_CREATED;
-		} else if (tinfo->tvcpu->state & 
+		} else if (state &
 			  (VMM_VCPU_STATE_READY | VMM_VCPU_STATE_RUNNING)) {
 			rc = VMM_THREAD_STATE_RUNNING;
-		} else if (tinfo->tvcpu->state & VMM_VCPU_STATE_PAUSED) {
+		} else if (state & VMM_VCPU_STATE_PAUSED) {
 			rc = VMM_THREAD_STATE_SLEEPING;
-		} else if (tinfo->tvcpu->state & VMM_VCPU_STATE_HALTED) {
+		} else if (state & VMM_VCPU_STATE_HALTED) {
 			rc = VMM_THREAD_STATE_STOPPED;
 		} else {
 			rc = -1;
@@ -152,22 +154,57 @@ int vmm_threads_get_state(struct vmm_thread *tinfo)
 	return rc;
 }
 
+int vmm_threads_get_hcpu(struct vmm_thread *tinfo, u32 *hcpu)
+{
+	if (!tinfo || !hcpu) {
+		return VMM_EFAIL;
+	}
+
+	return vmm_manager_vcpu_get_hcpu(tinfo->tvcpu, hcpu);
+}
+
+int vmm_thread_set_hcpu(struct vmm_thread *tinfo, u32 hcpu)
+{
+	if (!tinfo) {
+		return VMM_EFAIL;
+	}
+
+	return vmm_manager_vcpu_set_hcpu(tinfo->tvcpu, hcpu);
+}
+
+const struct vmm_cpumask *vmm_threads_get_affinity(struct vmm_thread *tinfo)
+{
+	if (!tinfo) {
+		return NULL;
+	}
+
+	return vmm_manager_vcpu_get_affinity(tinfo->tvcpu);
+}
+
+int vmm_threads_set_affinity(struct vmm_thread *tinfo,
+			     const struct vmm_cpumask *cpu_mask)
+{
+	if (!tinfo || !cpu_mask) {
+		return VMM_EFAIL;
+	}
+
+	return vmm_manager_vcpu_set_affinity(tinfo->tvcpu, cpu_mask);
+}
+
 struct vmm_thread *vmm_threads_id2thread(u32 tid)
 {
 	bool found;
 	irq_flags_t flags;
-	struct dlist *l;
-	struct vmm_thread *ret;
+	struct vmm_thread *tinfo;
 
-	ret = NULL;
+	tinfo = NULL;
 	found = FALSE;
 
 	/* Lock threads control */
 	vmm_spin_lock_irqsave(&thctrl.lock, flags);
 
-	list_for_each(l, &thctrl.thread_list) {
-		ret = list_entry(l, struct vmm_thread, head);
-		if (ret->tvcpu->id == tid) {
+	list_for_each_entry(tinfo, &thctrl.thread_list, head) {
+		if (tinfo->tvcpu->id == tid) {
 			found = TRUE;
 			break;
 		}
@@ -180,28 +217,26 @@ struct vmm_thread *vmm_threads_id2thread(u32 tid)
 		return NULL;
 	}
 
-	return ret;
+	return tinfo;
 }
 
 struct vmm_thread *vmm_threads_index2thread(int index)
 {
 	bool found;
 	irq_flags_t flags;
-	struct dlist *l;
-	struct vmm_thread *ret;
+	struct vmm_thread *tinfo;
 
 	if (index < 0) {
 		return NULL;
 	}
 
-	ret = NULL;
+	tinfo = NULL;
 	found = FALSE;
 
 	/* Lock threads control */
 	vmm_spin_lock_irqsave(&thctrl.lock, flags);
 
-	list_for_each(l, &thctrl.thread_list) {
-		ret = list_entry(l, struct vmm_thread, head);
+	list_for_each_entry(tinfo, &thctrl.thread_list, head) {
 		if (!index) {
 			found = TRUE;
 			break;
@@ -216,7 +251,7 @@ struct vmm_thread *vmm_threads_index2thread(int index)
 		return NULL;
 	}
 
-	return ret;
+	return tinfo;
 }
 
 u32 vmm_threads_count(void)
@@ -248,20 +283,27 @@ static void vmm_threads_entry(void)
 	
 	/* Nothing else to do for this thread.
 	 * Let us hope someone else will destroy it.
-	 * For now just hang. :( :( 
+	 * For now just hang. :( :(
          */
 	vmm_hang();
 }
 
-struct vmm_thread *vmm_threads_create(const char *thread_name, 
-				      int (*thread_fn) (void *udata),
-				      void *thread_data,
-				      u8 thread_priority,
-				      u64 thread_nsecs)
+struct vmm_thread *vmm_threads_create_rt(const char *thread_name,
+					 int (*thread_fn) (void *udata),
+					 void *thread_data,
+					 u8 thread_priority,
+				         u64 thread_nsecs,
+					 u64 thread_deadline,
+					 u64 thread_periodicity)
 {
 	irq_flags_t flags;
 	struct vmm_thread *tinfo;
 
+	/* Sanity check */
+	if (!thread_name || !thread_fn) {
+		return NULL;
+	}
+
 	/* Create thread structure instance */
 	tinfo = vmm_malloc(sizeof(struct vmm_thread));
 	if (!tinfo) {
@@ -270,12 +312,23 @@ struct vmm_thread *vmm_threads_create(const char *thread_name,
 	tinfo->tfn = thread_fn;
 	tinfo->tdata = thread_data;
 	tinfo->tnsecs = thread_nsecs;
+	if (tinfo->tnsecs == 0) {
+		tinfo->tnsecs = VMM_THREAD_DEF_TIME_SLICE;
+	}
+	tinfo->tdeadline = thread_deadline;
+	if (tinfo->tdeadline < tinfo->tnsecs) {
+		tinfo->tdeadline = tinfo->tnsecs;
+	}
+	tinfo->tperiodicity = thread_periodicity;
+	if (tinfo->tperiodicity < tinfo->tdeadline) {
+		tinfo->tperiodicity = tinfo->tdeadline;
+	}
 
 	/* Create an orphan vcpu for this thread */
 	tinfo->tvcpu = vmm_manager_vcpu_orphan_create(thread_name,
 			(virtual_addr_t)&vmm_threads_entry,
-			CONFIG_THREAD_STACK_SIZE,
-			thread_priority, thread_nsecs);
+			CONFIG_THREAD_STACK_SIZE, thread_priority,
+			thread_nsecs, thread_deadline, thread_periodicity);
 	if (!tinfo->tvcpu) {
 		vmm_free(tinfo);
 		return NULL;
