@@ -27,7 +27,7 @@ set xvisor_prompt "XVisor#"
 set arm_prompt "basic#"
 
 # start the test 
-spawn qemu-system-arm -M realview-pb-a8 -display none -serial stdio -kernel $qemu_img
+spawn qemu-system-arm -M realview-pb-a8 -m 256M -display none -serial stdio -kernel $qemu_img
 
 expect $xvisor_prompt
 send -- "help\r"
@@ -59,7 +59,7 @@ expect $xvisor_prompt
 
 set reset_out $expect_out(buffer)
 #puts $reset_out
-if { [string first "Initialize Board Final" $reset_out] > -1 } {
+if { [string first "init: board final" $reset_out] > -1 } {
         puts "The reset Command passed \n :: RESET TESTCASE PASS :: \n\n"
 } else {
         puts "The reset Command Failed \n :: RESET TESTCASE FAIL :: \n\n"
@@ -161,17 +161,6 @@ if { [string first "/" $devtree_chpath_out] > -1 } {
         puts "The devtree chpath Command Failed \n :: DEVTREE CHPATH TESTCASE FAIL :: \n\n"
 }
 
-send -- "devtree attrib /host/cpus\r"
-expect $xvisor_prompt
-
-set devtree_attrib_out $expect_out(buffer)
-#puts $devtree_attrib_out
-if { [string first "cpu_freq_mhz" $devtree_attrib_out] > -1 } {
-        puts "The devtree attrib Command passed \n :: DEVTREE ATTRIB TESTCASE PASS :: \n\n"
-} else {
-        puts "The devtree attrib Command Failed \n :: DEVTREE ATTRIB TESTCASE FAIL :: \n\n"
-}
-
 send -- "devtree print /\r"
 expect $xvisor_prompt
 
