@@ -23,10 +23,12 @@
 
 #include <arm_types.h>
 #include <arm_io.h>
+#include <arm_math.h>
+#include <arm_string.h>
 #include <arm_board.h>
 #include <arm_plat.h>
-#include <arm_string.h>
 #include <pic/gic.h>
+#include <timer/generic_timer.h>
 #include <serial/pl01x.h>
 
 void arm_board_reset(void)
@@ -64,12 +66,6 @@ u32 arm_board_linux_machine_type(void)
 void arm_board_linux_default_cmdline(char *cmdline, u32 cmdline_sz)
 {
 	arm_strcpy(cmdline, "root=/dev/ram rw earlyprintk console=ttyAMA0");
-	/* VirtIO Network Device */
-	arm_strcat(cmdline, " virtio_mmio.device=4K@0x40100000:50");
-	/* VirtIO Block Device */
-	arm_strcat(cmdline, " virtio_mmio.device=4K@0x40200000:51");
-	/* VirtIO Console Device */
-	arm_strcat(cmdline, " virtio_mmio.device=4K@0x40300000:52");
 }
 
 u32 arm_board_flash_addr(void)
@@ -163,6 +159,41 @@ int arm_board_pic_unmask(u32 irq)
 	return gic_unmask(0, irq);
 }
 
+void arm_board_timer_enable(void)
+{
+	return generic_timer_enable();
+}
+
+void arm_board_timer_disable(void)
+{
+	return generic_timer_disable();
+}
+
+u64 arm_board_timer_irqcount(void)
+{
+	return generic_timer_irqcount();
+}
+
+u64 arm_board_timer_irqdelay(void)
+{
+	return generic_timer_irqdelay();
+}
+
+u64 arm_board_timer_timestamp(void)
+{
+	return generic_timer_timestamp();
+}
+
+void arm_board_timer_change_period(u32 usecs)
+{
+	return generic_timer_change_period(usecs);
+}
+
+int arm_board_timer_init(u32 usecs)
+{
+	return generic_timer_init(usecs, 27);
+}
+
 #define	CA15X4_UART_BASE		V2M_UART0
 #define	CA15X4_UART_TYPE		PL01X_TYPE_1
 #define	CA15X4_UART_INCLK		24000000
